@@ -47,12 +47,15 @@
 #include "nsISOAPEncoding.h"
 #include "nsIComponentManager.h"
 #include "nsVoidArray.h"
-#include "nsSOAPUtils.h"
+
+#define NS_SOAP_1_1_ENCODING_NAMESPACE \
+   "http://schemas.xmlsoap.org/soap/encoding/"
+#define NS_SOAP_1_2_ENCODING_NAMESPACE \
+   "http://www.w3.org/2001/09/soap-encoding"
 
 WSPProxy::WSPProxy()
   : mIID(nsnull)
 {
-  NS_INIT_ISUPPORTS();
 }
 
 WSPProxy::~WSPProxy()
@@ -62,7 +65,8 @@ WSPProxy::~WSPProxy()
 NS_IMETHODIMP
 WSPProxy::Init(nsIWSDLPort* aPort,
                nsIInterfaceInfo* aPrimaryInterface,
-               const nsAReadableString& aQualifier,
+               nsIInterfaceInfoManager* aInterfaceInfoManager,
+               const nsAString& aQualifier,
                PRBool aIsAsync)
 {
   NS_ENSURE_ARG(aPort);
@@ -70,14 +74,27 @@ WSPProxy::Init(nsIWSDLPort* aPort,
 
   mPort = aPort;
   mPrimaryInterface = aPrimaryInterface;
+  mInterfaceInfoManager = aInterfaceInfoManager;
   mPrimaryInterface->GetIIDShared(&mIID);
   mQualifier.Assign(aQualifier);
   mIsAsync = aIsAsync;
+
+  nsresult rv;
+
+  mInterfaces = do_CreateInstance(NS_SCRIPTABLE_INTERFACES_CONTRACTID, &rv);
+  if (!mInterfaces) {
+    return rv;
+  }
+  
+  rv = mInterfaces->SetManager(mInterfaceInfoManager);
+  if (NS_FAILED(rv)) {
+    return rv;
+  }
   
   if (mIsAsync) {
     // Get the completion method info
     const nsXPTMethodInfo* listenerGetter;
-    nsresult rv = mPrimaryInterface->GetMethodInfo(3, &listenerGetter);
+    rv = mPrimaryInterface->GetMethodInfo(3, &listenerGetter);
     if (NS_FAILED(rv)) {
       return NS_ERROR_FAILURE;
     }
@@ -172,6 +189,7 @@ WSPProxy::CallMethod(PRUint16 methodIndex,
   else {
     methodOffset = 3;
   }
+
   nsCOMPtr<nsIWSDLOperation> operation;
   rv = mPort->GetOperation(methodIndex - methodOffset, 
                            getter_AddRefs(operation));
@@ -250,19 +268,8 @@ WSPProxy::CallMethod(PRUint16 methodIndex,
   // Set up the parameters to the call
   PRUint32 i, partCount;
   input->GetPartCount(&partCount);
-
-  PRUint32 parameterOrderCount;
-  operation->GetParameterOrderCount(&parameterOrderCount);
-
-  // The whichParam array stores the index of the parameter that
-  // corresponds to each part. The exception to there being a
-  // direct correspondence between part and parameter indices is
-  // the existence of a parameterOrder attribute for the operation
-  // or parameters that are arrays (and have lengths associated
-  // with them).
-#define MAX_PARTS 128
-  PRUint32 whichParam[MAX_PARTS];
-  PRUint32 arrayOffset = 0;
+  
+  PRUint32 maxParamIndex = info->GetParamCount()-1;
 
   // Iterate through the parts to figure out how many are
   // body vs. header blocks
@@ -277,25 +284,6 @@ WSPProxy::CallMethod(PRUint16 methodIndex,
     nsAutoString paramName;
     part->GetName(paramName);
 
-    // If the part is an array, then 
-    if (IsArray(part)) {
-      arrayOffset++;
-    }
-
-    // If there's a parameter count, then the order of the incoming
-    // parameter may be different than the part order.
-    if (parameterOrderCount) {
-      PRUint32 pindex;
-      rv = operation->GetParameterIndex(paramName, &pindex);
-      if (NS_FAILED(rv)) {
-        return NS_ERROR_FAILURE;
-      }
-      whichParam[i] = pindex + arrayOffset;
-    }
-    else {
-      whichParam[i] = i + arrayOffset;
-    }
-
     rv = part->GetBinding(getter_AddRefs(binding));
     if (NS_FAILED(rv)) {
       return rv;
@@ -354,8 +342,8 @@ WSPProxy::CallMethod(PRUint16 methodIndex,
   }
 
   // Now iterate through the parameters and set up the parameter blocks
-  PRUint32 bodyEntry = 0, headerEntry = 0;
-  for (i = 0; i < partCount; i++) {
+  PRUint32 bodyEntry = 0, headerEntry = 0, paramIndex = 0;
+  for (i = 0; i < partCount; paramIndex++, i++) {
     input->GetPart(i, getter_AddRefs(part));
     part->GetBinding(getter_AddRefs(binding));
     nsCOMPtr<nsISOAPPartBinding> partBinding = do_QueryInterface(binding);
@@ -429,19 +417,27 @@ WSPProxy::CallMethod(PRUint16 methodIndex,
       }
     }
 
-    const nsXPTParamInfo& paramInfo = info->GetParam(whichParam[i]);
-    PRUint32 arrayLength = 0;
-    if (IsArray(part)) {
-      arrayLength = params[whichParam[i]-1].val.u32;
-#ifdef DEBUG
-      const nsXPTType& arrayType = paramInfo.GetType();
-      NS_ASSERTION(arrayType.IsArray(), "WSPProxy:: array type incorrect");
-#endif
+    // Look ahead in the param info array to see if the current part has to be
+    // treated as an array. If so then get the array length from the current 
+    // param and increment the param index.
+
+    PRUint32 arrayLength;
+
+    if (paramIndex < maxParamIndex &&
+        info->GetParam((PRUint8)(paramIndex+1)).GetType().IsArray()) {
+      arrayLength = params[paramIndex++].val.u32;
     }
+    else {
+      arrayLength = 0;
+    }
+
+    NS_ASSERTION(paramIndex <= maxParamIndex, "WSDL/IInfo param count mismatch");
+    
+    const nsXPTParamInfo& paramInfo = info->GetParam(paramIndex);
 
     nsCOMPtr<nsIVariant> value;
     rv = ParameterToVariant(mPrimaryInterface, methodIndex,
-                            &paramInfo, params[whichParam[i]], 
+                            &paramInfo, params[paramIndex], 
                             arrayLength, getter_AddRefs(value));
     if (NS_FAILED(rv)) {
       goto call_method_end;
@@ -460,8 +456,7 @@ WSPProxy::CallMethod(PRUint16 methodIndex,
   }
 
   WSPCallContext* ccInst;
-  ccInst = new WSPCallContext(this, call, methodName, 
-                              operation);
+  ccInst = new WSPCallContext(this, call, methodName, operation);
   if (!ccInst) {
     rv = NS_ERROR_OUT_OF_MEMORY;
     goto call_method_end;
@@ -502,7 +497,7 @@ WSPProxy::CallMethod(PRUint16 methodIndex,
     *retval = cc;
     NS_ADDREF(*retval);
     
-    rv = ccInst->CallAsync(methodIndex-1, mAsyncListener);
+    rv = ccInst->CallAsync(methodIndex, mAsyncListener);
     if (NS_FAILED(rv)) {
       goto call_method_end;
     }
@@ -524,64 +519,6 @@ call_method_end:
   return rv;
 }
 
-PRBool
-WSPProxy::IsArray(nsIWSDLPart* aPart)
-{
-  nsCOMPtr<nsISchemaComponent> schemaComponent;
-  nsresult rv = aPart->GetSchemaComponent(getter_AddRefs(schemaComponent));
-  if (NS_FAILED(rv)) {
-    return PR_FALSE;
-  }
-
-  nsCOMPtr<nsISchemaType> type;
-  nsCOMPtr<nsISchemaElement> element = do_QueryInterface(schemaComponent);
-  if (element) {
-    rv = element->GetType(getter_AddRefs(type));
-    if (NS_FAILED(rv)) {
-      return PR_FALSE;
-    }
-  }
-  else {
-    type = do_QueryInterface(schemaComponent, &rv);
-    if (NS_FAILED(rv)) {
-      return PR_FALSE;
-    }
-  }
-
-  nsCOMPtr<nsISchemaComplexType> complexType(do_QueryInterface(type));
-  if (!complexType) {
-    return PR_FALSE;
-  }
-
-  while (complexType) {
-    PRUint16 derivation;
-    complexType->GetDerivation(&derivation);
-    if (derivation == nsISchemaComplexType::DERIVATION_SELF_CONTAINED) {
-      break;
-    }
-    nsCOMPtr<nsISchemaType> base;
-    complexType->GetBaseType(getter_AddRefs(base));
-    complexType = do_QueryInterface(base);
-  }
-  
-  // If the base type is not a complex type, then we're done
-  if (!complexType) {
-    return PR_FALSE;
-  }
-
-  nsAutoString name, ns;
-  complexType->GetName(name);
-  complexType->GetTargetNamespace(ns);
-
-  if (name.Equals(NS_LITERAL_STRING("Array")) &&
-      (ns.Equals(*nsSOAPUtils::kSOAPEncURI[nsISOAPMessage::VERSION_1_1]) ||
-       ns.Equals(*nsSOAPUtils::kSOAPEncURI[nsISOAPMessage::VERSION_1_2]))) {
-    return PR_TRUE;
-  }
-
-  return PR_FALSE;
-}
-
 nsresult
 WSPProxy::ParameterToVariant(nsIInterfaceInfo* aInterfaceInfo,
                              PRUint32 aMethodIndex,
@@ -656,6 +593,17 @@ WSPProxy::XPTCMiniVariantToVariant(uint8 aTypeTag,
 {
   nsresult rv;
 
+  // If a variant is passed in, just return it as is.
+  if (aTypeTag == nsXPTType::T_INTERFACE) {
+    nsISupports* inst = (nsISupports*)aResult.val.p;
+    nsCOMPtr<nsIVariant> instVar = do_QueryInterface(inst);
+    if (instVar) {
+      *aVariant = instVar;
+      NS_ADDREF(*aVariant);
+      return NS_OK;
+    }
+  }
+
   nsCOMPtr<nsIWritableVariant> var = do_CreateInstance(NS_VARIANT_CONTRACTID, 
                                                        &rv);
   if (NS_FAILED(rv)) {
@@ -702,6 +650,12 @@ WSPProxy::XPTCMiniVariantToVariant(uint8 aTypeTag,
     case nsXPTType::T_WCHAR:
       var->SetAsWChar(aResult.val.wc);
       break;
+    case nsXPTType::T_CHAR_STR:
+      var->SetAsString(NS_STATIC_CAST(char*, aResult.val.p));
+      break;
+    case nsXPTType::T_WCHAR_STR:
+      var->SetAsWString(NS_STATIC_CAST(PRUnichar*, aResult.val.p));
+      break;
     case nsXPTType::T_DOMSTRING:
       var->SetAsAString(*((nsAString*)aResult.val.p));
       break;
@@ -753,60 +707,58 @@ WSPProxy::ArrayXPTCMiniVariantToVariant(uint8 aTypeTag,
     PRUint32 i = 0;
     void* array = aResult.val.p;
     void* entries;
+    nsISupports** entriesSup = nsnull;
     const nsIID* iid = nsnull; 
 
-#define POPULATE(_t,_v)                                                  \
-  PR_BEGIN_MACRO                                                         \
-    _t* entries##_v = (_t*)nsMemory::Alloc(aLength * sizeof(_t));        \
-    if (!entries##_v) {                                                  \
-      return NS_ERROR_OUT_OF_MEMORY;                                     \
-    }                                                                    \
-    for(i = 0; i < aLength; i++)                                         \
-    {                                                                    \
-      entries##_v[i] = *((_t*)array + i);                                \
-    }                                                                    \
-    entries = (void*)entries##_v;                                        \
-  PR_END_MACRO
-
     switch (aTypeTag) {
-      case nsXPTType::T_I8:            POPULATE(PRInt8, i8);      break; 
-      case nsXPTType::T_I16:           POPULATE(PRInt16, i16);    break;
-      case nsXPTType::T_I32:           POPULATE(PRInt32, i32);    break;
-      case nsXPTType::T_I64:           POPULATE(PRInt64, i64);    break;
-      case nsXPTType::T_U8:            POPULATE(PRUint8, u8);     break; 
-      case nsXPTType::T_U16:           POPULATE(PRUint16, u16);   break;
-      case nsXPTType::T_U32:           POPULATE(PRUint32, u32);   break;
-      case nsXPTType::T_U64:           POPULATE(PRUint64, u64);   break;
-      case nsXPTType::T_FLOAT:         POPULATE(float, f);        break;
-      case nsXPTType::T_DOUBLE:        POPULATE(double, d);       break;
-      case nsXPTType::T_BOOL:          POPULATE(PRBool, b);       break;
-      case nsXPTType::T_CHAR:          POPULATE(char, c);         break;
-      case nsXPTType::T_WCHAR:         POPULATE(PRUnichar, wc);   break;
+      case nsXPTType::T_I8:
+      case nsXPTType::T_I16:
+      case nsXPTType::T_I32:
+      case nsXPTType::T_I64:
+      case nsXPTType::T_U8:
+      case nsXPTType::T_U16:
+      case nsXPTType::T_U32:
+      case nsXPTType::T_U64:
+      case nsXPTType::T_FLOAT:
+      case nsXPTType::T_DOUBLE:
+      case nsXPTType::T_BOOL:
+      case nsXPTType::T_CHAR:
+      case nsXPTType::T_WCHAR:
+      case nsXPTType::T_CHAR_STR:
+      case nsXPTType::T_WCHAR_STR:
+        entries = array;
+        break;
       case nsXPTType::T_INTERFACE:  
       {
-        nsISupports** entriesSup = (nsISupports**)nsMemory::Alloc(aLength * 
-                                                        sizeof(nsISupports*));
-        if (!entriesSup) {
-          return NS_ERROR_OUT_OF_MEMORY;
+        aInterfaceInfo->GetIIDShared(&iid);
+        if (iid->Equals(NS_GET_IID(nsIVariant))) {
+          entries = array;
         }
-        const nsIID& propbagIID = NS_GET_IID(nsIPropertyBag);
-        iid = &propbagIID;
-        entries = (void*)entriesSup;
-        for(i = 0; i < aLength; i++) {
-          nsISupports* instance = *((nsISupports**)array + i);
-          nsISupports** outptr = entriesSup + i;
-          if (instance) {
-            nsCOMPtr<nsIPropertyBag> propBag;
-            rv = WrapInPropertyBag(instance, aInterfaceInfo,
-                                   getter_AddRefs(propBag));
-            if (NS_FAILED(rv)) {
-              break;
-            }
-            propBag->QueryInterface(NS_GET_IID(nsISupports),
-                                    (void**)outptr);
+        else {
+          entriesSup = (nsISupports**)nsMemory::Alloc(aLength * 
+                                                      sizeof(nsISupports*));
+          if (!entriesSup) {
+            return NS_ERROR_OUT_OF_MEMORY;
           }
-          else {
-            *outptr = nsnull;
+          const nsIID& propbagIID = NS_GET_IID(nsIPropertyBag);
+          iid = &propbagIID;
+          entries = (void*)entriesSup;
+          for(i = 0; i < aLength; i++) {
+            nsISupports* instance = *((nsISupports**)array + i);
+            nsISupports** outptr = entriesSup + i;
+            if (instance) {
+              nsCOMPtr<nsIPropertyBag> propBag;
+              rv = WrapInPropertyBag(instance, aInterfaceInfo,
+                                     getter_AddRefs(propBag));
+              if (NS_FAILED(rv)) {
+                break;
+              }
+              propBag->QueryInterface(NS_GET_IID(nsISupports),
+                                      (void**)outptr);
+            }
+            else {
+              *outptr = nsnull;
+            }
           }
         }
         aTypeTag = nsXPTType::T_INTERFACE_IS;
@@ -817,17 +769,12 @@ WSPProxy::ArrayXPTCMiniVariantToVariant(uint8 aTypeTag,
         return NS_ERROR_FAILURE;
     }
 
-#undef POPULATE
-
     if (NS_SUCCEEDED(rv)) {
       rv = retvar->SetAsArray(aTypeTag, iid, aLength, entries);
     }
 
-    if (aTypeTag == nsXPTType::T_INTERFACE) {
-      NS_FREE_XPCOM_ISUPPORTS_POINTER_ARRAY(i, (nsISupports**)entries);
-    }
-    else {
-      nsMemory::Free(entries);
+    if (entriesSup) {
+      NS_FREE_XPCOM_ISUPPORTS_POINTER_ARRAY(i, entriesSup);
     }
   }
   else {
@@ -1004,16 +951,28 @@ WSPProxy::VariantToValue(uint8 aTypeTag,
     case nsXPTType::T_WCHAR:
       rv = aProperty->GetAsWChar((PRUnichar*)aValue);
       break;
+    case nsXPTType::T_CHAR_STR:
+      rv = aProperty->GetAsString((char**)aValue);
+      break;
+    case nsXPTType::T_WCHAR_STR:
+      rv = aProperty->GetAsWString((PRUnichar**)aValue);
+      break;
     case nsXPTType::T_DOMSTRING:
       rv = aProperty->GetAsAString(*(nsAString*)aValue);
       break;
     case nsXPTType::T_INTERFACE:
     {
+      const nsIID* iid;
+      aInterfaceInfo->GetIIDShared(&iid);
       PRUint16 dataType;
       aProperty->GetDataType(&dataType);
       if (dataType == nsIDataType::VTYPE_EMPTY) {
         *(nsISupports**)aValue = nsnull;
       }
+      else if (iid->Equals(NS_GET_IID(nsIVariant))) {
+        *(nsIVariant**)aValue = aProperty;
+        NS_ADDREF(*(nsIVariant**)aValue);
+      }
       else {
         nsCOMPtr<nsISupports> sup;
         rv = aProperty->GetAsISupports(getter_AddRefs(sup));
@@ -1031,8 +990,6 @@ WSPProxy::VariantToValue(uint8 aTypeTag,
           return rv;
         }
 
-        const nsIID* iid;
-        aInterfaceInfo->GetIIDShared(&iid);
         rv = wrapper->QueryInterface(*iid, (void**)aValue);
       }
       break;
@@ -1076,45 +1033,52 @@ WSPProxy::VariantToArrayValue(uint8 aTypeTag,
     case nsXPTType::T_BOOL:
     case nsXPTType::T_CHAR:
     case nsXPTType::T_WCHAR:
+    case nsXPTType::T_CHAR_STR:
+    case nsXPTType::T_WCHAR_STR:
       *((void**)aResult[1].val.p) = array;
       break;
     case nsXPTType::T_INTERFACE:
     case nsXPTType::T_INTERFACE_IS:
     {
-      if (!arrayIID.Equals(NS_GET_IID(nsIPropertyBag))) {
+      if (arrayIID.Equals(NS_GET_IID(nsIVariant))) {
+        *((void**)aResult[1].val.p) = array;
+      }
+      else if (!arrayIID.Equals(NS_GET_IID(nsIPropertyBag))) {
         NS_ERROR("Array of complex types should be represented by property bags");
         return NS_ERROR_FAILURE;
       }
-      nsISupports** outptr = (nsISupports**)nsMemory::Alloc(count * 
-                                                         sizeof(nsISupports*));
-      if (!outptr) {
-        return NS_ERROR_OUT_OF_MEMORY;
-      }
-      nsISupports** arraySup = (nsISupports**)array;
-      const nsIID* iid;
-      aInterfaceInfo->GetIIDShared(&iid);
-      PRUint32 i;
-      for (i = 0; i < count; i++) {
-        nsCOMPtr<nsIPropertyBag> propBag(do_QueryInterface(arraySup[i]));
-        if (!propBag) {
-          *(outptr + i) = nsnull;
+      else {
+        nsISupports** outptr = (nsISupports**)nsMemory::Alloc(count * 
+                                                              sizeof(nsISupports*));
+        if (!outptr) {
+          return NS_ERROR_OUT_OF_MEMORY;
         }
-        else {
-          nsCOMPtr<nsISupports> wrapper;
-          rv = WrapInComplexType(propBag, aInterfaceInfo, 
-                                 getter_AddRefs(wrapper));
-          if (NS_FAILED(rv)) {
-            NS_FREE_XPCOM_ISUPPORTS_POINTER_ARRAY(i, outptr);
-            return rv;
+        nsISupports** arraySup = (nsISupports**)array;
+        const nsIID* iid;
+        aInterfaceInfo->GetIIDShared(&iid);
+        PRUint32 i;
+        for (i = 0; i < count; i++) {
+          nsCOMPtr<nsIPropertyBag> propBag(do_QueryInterface(arraySup[i]));
+          if (!propBag) {
+            *(outptr + i) = nsnull;
           }
-          rv = wrapper->QueryInterface(*iid, (void**)(outptr + i));
-          if (NS_FAILED(rv)) {
-            NS_FREE_XPCOM_ISUPPORTS_POINTER_ARRAY(i, outptr);
-            return rv;
+          else {
+            nsCOMPtr<nsISupports> wrapper;
+            rv = WrapInComplexType(propBag, aInterfaceInfo, 
+                                   getter_AddRefs(wrapper));
+            if (NS_FAILED(rv)) {
+              NS_FREE_XPCOM_ISUPPORTS_POINTER_ARRAY(i, outptr);
+              return rv;
+            }
+            rv = wrapper->QueryInterface(*iid, (void**)(outptr + i));
+            if (NS_FAILED(rv)) {
+              NS_FREE_XPCOM_ISUPPORTS_POINTER_ARRAY(i, outptr);
+              return rv;
+            }
           }
         }
+        *((void**)aResult[1].val.p) = outptr;
       }
-      *((void**)aResult[1].val.p) = outptr;
       break;
     }
     default:
@@ -1179,6 +1143,60 @@ WSPProxy::GetPendingCalls(nsISimpleEnumerator * *aPendingCalls)
   return NS_ERROR_NOT_IMPLEMENTED;
 }
 
+nsresult WSPProxy::GetInterfaceName(PRBool listener, char** retval)
+{
+  if (!mPrimaryInterface) {
+    return NS_ERROR_FAILURE;
+  }
+
+  const char* rawName;
+  nsresult rv = mPrimaryInterface->GetNameShared(&rawName);
+  if (NS_FAILED(rv)) {
+    return rv;
+  }
+
+  nsCAutoString name;
+
+  if (listener) {
+    if (mIsAsync) {
+      name.Assign(rawName, strlen(rawName) - (sizeof("Async")-1) );
+    }
+    else {
+      name.Assign(rawName);
+    }
+    name.Append("Listener");
+  }
+  else {
+    name.Assign(rawName);
+  }
+
+  *retval = (char*) nsMemory::Clone(name.get(), name.Length()+1);
+  return *retval ? NS_OK : NS_ERROR_OUT_OF_MEMORY;
+}
+
+/* readonly attribute string primaryInterfaceName; */
+NS_IMETHODIMP 
+WSPProxy::GetPrimaryInterfaceName(char * *aPrimaryInterfaceName)
+{
+  return GetInterfaceName(PR_FALSE, aPrimaryInterfaceName);
+}
+
+/* readonly attribute string primaryAsyncListenerInterfaceName; */
+NS_IMETHODIMP 
+WSPProxy::GetPrimaryAsyncListenerInterfaceName(char * *aPrimaryAsyncListenerInterfaceName)
+{
+  return GetInterfaceName(PR_TRUE, aPrimaryAsyncListenerInterfaceName);
+}
+
+/* readonly attribute nsIScriptableInterfaces interfaces; */
+NS_IMETHODIMP 
+WSPProxy::GetInterfaces(nsIScriptableInterfaces * *aInterfaces)
+{
+  *aInterfaces = mInterfaces;
+  NS_IF_ADDREF(*aInterfaces);
+  return NS_OK;
+}
+
 ///////////////////////////////////////////////////
 //
 // Implementation of nsIClassInfo
