@@ -48,6 +48,7 @@
 #include "nsIPresContext.h"
 #include "nsIDOMNSHTMLInputElement.h"
 #include "nsIDOMNSHTMLTextAreaElement.h"
+#include "nsXPCOM.h"
 #include "nsISupportsPrimitives.h"
 #include "nsIDOMRange.h"
 
@@ -195,30 +196,27 @@ nsresult nsCopySupport::HTMLCopy(nsISelection *aSel, nsIDocument *aDoc, PRInt16
       }
       
       // get wStrings to hold clip data
-      nsCOMPtr<nsISupportsWString> dataWrapper, contextWrapper, infoWrapper;
-      dataWrapper = do_CreateInstance(NS_SUPPORTS_WSTRING_CONTRACTID);
+      nsCOMPtr<nsISupportsString> dataWrapper, contextWrapper, infoWrapper;
+      dataWrapper = do_CreateInstance(NS_SUPPORTS_STRING_CONTRACTID);
       NS_ENSURE_TRUE(dataWrapper, NS_ERROR_FAILURE);
       if (bIsHTMLCopy)
       {
-        contextWrapper = do_CreateInstance(NS_SUPPORTS_WSTRING_CONTRACTID);
+        contextWrapper = do_CreateInstance(NS_SUPPORTS_STRING_CONTRACTID);
         NS_ENSURE_TRUE(contextWrapper, NS_ERROR_FAILURE);
-        infoWrapper = do_CreateInstance(NS_SUPPORTS_WSTRING_CONTRACTID);
+        infoWrapper = do_CreateInstance(NS_SUPPORTS_STRING_CONTRACTID);
         NS_ENSURE_TRUE(infoWrapper, NS_ERROR_FAILURE);
       }
       
       // populate the strings
       nsresult data_rv = NS_OK, context_rv = NS_OK, info_rv = NS_OK;
       data_rv =
-        dataWrapper->SetDataWithLength(buffer.Length(),
-                                       NS_CONST_CAST(PRUnichar*, buffer.get()));
+        dataWrapper->SetData(buffer);
       if (bIsHTMLCopy)
       {
         context_rv =
-          contextWrapper->SetDataWithLength(parents.Length(),
-                                            NS_CONST_CAST(PRUnichar*, parents.get()));
+          contextWrapper->SetData(parents);
         info_rv =
-          infoWrapper->SetDataWithLength(info.Length(),
-                                         NS_CONST_CAST(PRUnichar*, info.get()));
+          infoWrapper->SetData(info);
       }
       
       // QI the data object an |nsISupports| so that when the transferable holds
