@@ -55,12 +55,12 @@
 #include "nsIDeviceContext.h"
 #include "nsIFontMetrics.h"
 #include "nsIImage.h"
-#include "nsStyleUtil.h"
 #include "nsStyleConsts.h"
 #include "nsFormControlFrame.h"
 #include "nsGUIEvent.h"
 #include "nsLayoutAtoms.h"
 #include "nsIServiceManager.h"
+#include "nsContainerFrame.h"
 #ifdef ACCESSIBILITY
 #include "nsIAccessibilityService.h"
 #endif
@@ -121,7 +121,7 @@ public:
 
   virtual void MouseClicked(nsIPresContext* aPresContext);
 
-  NS_IMETHOD GetType(PRInt32* aType) const;
+  NS_IMETHOD_(PRInt32) GetType() const;
 
   NS_IMETHOD GetName(nsAString* aName);
 
@@ -284,14 +284,15 @@ nsImageControlFrame::Init(nsIPresContext*  aPresContext,
     // the view's size is not know yet, but its size will be kept in synch with our frame.
     nsRect boundBox(0, 0, 0, 0); 
     result = view->Init(viewMan, boundBox, parView);
-    viewMan->SetViewContentTransparency(view, PR_TRUE);
+
+    nsContainerFrame::SyncFrameViewProperties(aPresContext, this, aContext, view);
+
+    // this gets reset during reflow anyway
+    // viewMan->SetViewContentTransparency(view, PR_TRUE);
+
     // XXX put the view last in document order until we know how to do better
     viewMan->InsertChild(parView, view, nsnull, PR_TRUE);
     SetView(aPresContext, view);
-
-    const nsStyleVisibility* vis = (const nsStyleVisibility*) mStyleContext->GetStyleData(eStyleStruct_Visibility);
-    // set the opacity
-    viewMan->SetViewOpacity(view, vis->mOpacity);
   }
 
   return rv;
@@ -391,21 +392,16 @@ nsImageControlFrame::GetTranslatedRect(nsIPresContext* aPresContext, nsRect& aRe
   aRect = nsRect(viewOffset.x, viewOffset.y, mRect.width, mRect.height);
 }
 
-NS_IMETHODIMP
-nsImageControlFrame::GetType(PRInt32* aType) const
+NS_IMETHODIMP_(PRInt32)
+nsImageControlFrame::GetType() const
 {
-  *aType = NS_FORM_INPUT_IMAGE;
-  return NS_OK;
+  return NS_FORM_INPUT_IMAGE;
 }
 
 NS_IMETHODIMP
 nsImageControlFrame::GetName(nsAString* aResult)
 {
-  if (nsnull == aResult) {
-    return NS_OK;
-  } else {
-    return nsFormControlHelper::GetName(mContent, aResult);
-  }
+  return nsFormControlHelper::GetName(mContent, aResult);
 }
 
 NS_IMETHODIMP
