@@ -51,15 +51,14 @@
 #include "nsISmtpServer.h"
 #include "nsISmtpService.h"
 #include "nsMsgCompCID.h"
-
-static NS_DEFINE_CID(kSmtpServiceCID, NS_SMTPSERVICE_CID);
-static NS_DEFINE_CID(kStandardUrlCID, NS_STANDARDURL_CID);
+#include "nsReadableUtils.h"
+#include "nsEscape.h"
+#include "prmem.h"
 
 NS_IMPL_ISUPPORTS1(nsSmtpDelegateFactory, nsIRDFDelegateFactory)
 
 nsSmtpDelegateFactory::nsSmtpDelegateFactory()
 {
-    NS_INIT_ISUPPORTS();
 }
 
 nsSmtpDelegateFactory::~nsSmtpDelegateFactory()
@@ -76,11 +75,8 @@ nsSmtpDelegateFactory::CreateDelegate(nsIRDFResource *aOuter,
   const char* uri;
   aOuter->GetValueConst(&uri);
 
-  nsCOMPtr<nsIURL> url;
-  rv = nsComponentManager::CreateInstance(kStandardUrlCID, nsnull,
-                                          NS_GET_IID(nsIURL),
-                                          (void **)getter_AddRefs(url));
-  if (NS_FAILED(rv)) return rv;
+  nsCOMPtr<nsIURL> url = do_CreateInstance(NS_STANDARDURL_CONTRACTID, &rv);
+  NS_ENSURE_SUCCESS(rv, rv);
   
   rv = url->SetSpec(nsDependentCString(uri));
 
@@ -94,11 +90,17 @@ nsSmtpDelegateFactory::CreateDelegate(nsIRDFResource *aOuter,
   url->GetAsciiHost(hostname);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  nsCOMPtr<nsISmtpService> smtpService = do_GetService(kSmtpServiceCID, &rv);
+  nsCOMPtr<nsISmtpService> smtpService = do_GetService(NS_SMTPSERVICE_CONTRACTID, &rv);
   NS_ENSURE_SUCCESS(rv, rv);
 
+  char *unescapedUserPass = ToNewCString(userpass);
+  if (!unescapedUserPass)
+    return NS_ERROR_OUT_OF_MEMORY;
+
+  nsUnescape(unescapedUserPass);
   nsCOMPtr<nsISmtpServer> smtpServer;
-  rv = smtpService->FindServer(userpass.get(), hostname.get(), getter_AddRefs(smtpServer));
+  rv = smtpService->FindServer(unescapedUserPass, hostname.get(), getter_AddRefs(smtpServer));
+  PR_FREEIF(unescapedUserPass);
 
   // no server, it's a failure!
   if (NS_FAILED(rv)) return rv;
